@@ -50,16 +50,24 @@ apply_pixel <- function(x, ...) {
 #' @param keep_bands logical; keep bands of input data cube, defaults to FALSE, i.e. original bands will be dropped
 #' @param ... not used
 #' @param FUN user-defined R function that is applied on all pixels (see Details)
+#' @param load_pkgs logical or character; if TRUE, all currently attached packages will be attached automatically before executing FUN in spawned R processes, specific packages can alternatively be provided as a character vector.
+#' @param load_env logical or environment; if TRUE, the current global environment will be restored automatically before executing FUN in spawned R processes, can be set to a custom environment.
 #' @return a proxy data cube object
 #' @details 
 #' 
 #' The function can either apply simple arithmetic C expressions given as a character vector (expr argument), or apply a custom R reducer function if FUN is provided.
 #' 
 #' In the former case, gdalcubes uses the \href{https://github.com/codeplea/tinyexpr}{tinyexpr library} to evaluate expressions in C / C++, you can look at the \href{https://github.com/codeplea/tinyexpr#functions-supported}{library documentation}
-#' to see what kind of expressions you can execute. Pixel band values can be accessed by name.
-#' 
+#' to see what kind of expressions you can execute. Pixel band values can be accessed by name. Predefined variables that can be used within the expression include integer pixel indexes (\code{ix}, \code{iy}, \code{it}), and 
+#' pixel coordinates (\code{left}, \code{right}, \code{top}, \code{bottom}), \code{t0}, \code{t1}), where the last two values are provided seconds since epoch time.
+#'  
 #' FUN receives values of the bands from one pixel as a (named) vector and should return a numeric vector with identical length for all pixels. Elements of the
-#' result vectors will be interpreted as bands in the result data cube.  
+#' result vectors will be interpreted as bands in the result data cube. Notice that by default, since FUN is executed in a separate
+#' R process, it cannot access any variables from outside and required packages must be loaded within FUN. To restore the current environment and
+#' automatically load packages, set \code{load_env} and/or \code{load_pkgs} to \code{TRUE}.
+#' 
+#' For more details and examples on how to write user-defined functions, please refer to the gdalcubes website 
+#' at \url{https://gdalcubes.github.io/source/concepts/udfs.html}.
 #' 
 #' @examples 
 #' # create image collection from example Landsat data only 
@@ -91,13 +99,14 @@ apply_pixel <- function(x, ...) {
 #'    })
 #' L8.ndvi.noisy
 #' 
+#' 
 #' \donttest{
 #' plot(L8.ndvi.noisy)
 #' }
 #'  
 #' @note This function returns a proxy object, i.e., it will not start any computations besides deriving the shape of the result.
 #' @export
-apply_pixel.cube <- function(x, expr, names=NULL, keep_bands=FALSE, ..., FUN) {
+apply_pixel.cube <- function(x, expr, names=NULL, keep_bands=FALSE, ..., FUN, load_pkgs=FALSE, load_env=FALSE) {
   stopifnot(is.cube(x))
   
   
@@ -147,6 +156,37 @@ apply_pixel.cube <- function(x, expr, names=NULL, keep_bands=FALSE, ..., FUN) {
       })
     }
     
+    
+    if (is.logical(load_env)) {
+      if (load_env) {
+        load_env = .GlobalEnv
+      }
+      else
+        load_env = NULL
+    }
+    if (!is.null(load_env)) {
+      if (!is.environment(load_env)) {
+        warning("Expected either FALSE/TRUE or environment for load_env; parameter will be set to FALSE.")
+        load_env = NULL
+      }
+    }
+    
+    if (is.logical(load_pkgs)) {
+      if (load_pkgs) {
+        load_pkgs = .packages()
+      }
+      else {
+        load_pkgs = NULL
+      }
+    }
+    if (!is.null(load_pkgs)) {
+      if (!is.character(load_pkgs)) {
+        warning("Expected either FALSE/TRUE or character vector for load_pkgs; parameter will be set to FALSE.")
+        load_pkgs = NULL
+      }
+    }
+    
+    
     # create src file
     # TODO: load the same packages as in the current workspace? see (.packages())
     funstr = serialize_function(FUN)
@@ -162,6 +202,17 @@ apply_pixel.cube <- function(x, expr, names=NULL, keep_bands=FALSE, ..., FUN) {
     cat(paste0(".libPaths(",  paste(deparse(.libPaths()),collapse=""), ")\n"), file = srcfile2, append = FALSE) 
     
     cat("require(gdalcubes)", "\n", file = srcfile2, append = TRUE)
+    if (!is.null(load_pkgs)) {
+      cat(paste0("require(", load_pkgs,")",collapse  = "\n"), "\n", file = srcfile2, append = TRUE) 
+    }
+    if (!is.null(load_env)) {
+      if (sum(sapply(ls(envir = load_env), FUN = function(x) {object.size(get(x, envir = load_env))})) > 100*1024^2) {
+        warning("The current environment seems to be rather large (> 100 Mb), if this results in reduced performance, please consider setting load_env = FALSE.")
+      }
+      envfile = tempfile(pattern="renv_", fileext = ".rda")
+      save(list = ls(envir = load_env),file = envfile, envir = load_env)
+      cat(paste0("load(\"", envfile, "\")"), "\n", file = srcfile2, append = TRUE)
+    }
     cat(paste("assign(\"f\", eval(parse(\"", srcfile1, "\")))", sep=""), "\n", file = srcfile2, append = TRUE)
     cat("write_chunk_from_array(apply_pixel(read_chunk_as_array(), f))", "\n", file = srcfile2, append = TRUE)
     cmd <- paste(file.path(R.home("bin"),"Rscript"), " --vanilla ", srcfile2, sep="")
