@@ -8,7 +8,7 @@
 #' if (!file.exists(file.path(tempdir(), "L8.db"))) {
 #'   L8_files <- list.files(system.file("L8NY18", package = "gdalcubes"),
 #'                          ".TIF", recursive = TRUE, full.names = TRUE)
-#'   create_image_collection(L8_files, "L8_L1TP", file.path(tempdir(), "L8.db")) 
+#'   create_image_collection(L8_files, "L8_L1TP", file.path(tempdir(), "L8.db"), quiet = TRUE) 
 #' }
 #' L8.col = image_collection(file.path(tempdir(), "L8.db"))
 #' v = cube_view(extent=list(left=388941.2, right=766552.4, 
@@ -90,7 +90,7 @@ is.slice_time_cube  <- function(obj) {
 #' if (!file.exists(file.path(tempdir(), "L8.db"))) {
 #'   L8_files <- list.files(system.file("L8NY18", package = "gdalcubes"),
 #'                          ".TIF", recursive = TRUE, full.names = TRUE)
-#'   create_image_collection(L8_files, "L8_L1TP", file.path(tempdir(), "L8.db")) 
+#'   create_image_collection(L8_files, "L8_L1TP", file.path(tempdir(), "L8.db"), quiet = TRUE) 
 #' }
 #' L8.col = image_collection(file.path(tempdir(), "L8.db"))
 #' v = cube_view(extent=list(left=388941.2, right=766552.4, 
@@ -126,12 +126,12 @@ slice_space <- function(cube, loc=NULL, i=NULL) {
   }
   
   if (!is.null(i)) {
-    if (i %% 1 != 0) {
-      stop("Invalid argument: i must be exactly two integer number")
-    }
     if (length(i) != 2) {
       stop("Invalid argument: length(i) is not equal to 2")
     }
+    if (any(i %% 1 != 0)) {
+      stop("Invalid argument: i must be integer numbers")
+    }
     x = gc_create_slice_space_cube(cube, numeric(0), as.integer(i))
   }
   else {
