@@ -25,7 +25,7 @@
 #ifndef IMAGE_COLLECTION_H
 #define IMAGE_COLLECTION_H
 
-#include <ogr_spatialref.h>
+//#include <ogr_spatialref.h>
 
 #include "collection_format.h"
 #include "coord_types.h"
