@@ -1,7 +1,7 @@
 /*
     MIT License
 
-    Copyright (c) 2019 Marius Appel <marius.appel@uni-muenster.de>
+    Copyright (c) 2019 Marius Appel <marius.appel@hs-bochum.de>
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to deal
@@ -79,39 +79,14 @@ struct progress_none : public progress {
 */
 struct progress_simple_stdout : public progress {
     std::shared_ptr<progress> get() override { return std::make_shared<progress_simple_stdout>(); }
-    void set(double p) override {
-        _m.lock();
-        _set(p);
-        _m.unlock();
-    };
-    void increment(double dp) override {
-        _m.lock();
-        _set(_p + dp);
-        _m.unlock();
-    }
-    virtual void finalize() override {
-        _m.lock();
-        for (uint16_t i = 0; i < (((int)(100 * _p)) / 10); ++i) {
-            std::cout << "=";
-        }
-        std::cout << ">| DONE." << std::endl;
-        _m.unlock();
-    }
+    void set(double p) override;
+    void increment(double dp) override;
+    virtual void finalize() override;
 
     progress_simple_stdout() : _p(0) {}
 
    private:
-    void _set(double p) {  // not synchronized
-        _m.lock();
-        _p = p;
-        for (uint16_t i = 0; i < (((int)(100 * p)) / 10); ++i) {
-            std::cout << "=";
-        }
-        std::cout << "> (" << std::round(100 * p) << "%)";
-        std::cout << "\r";
-        std::cout.flush();
-        _m.unlock();
-    };
+    void _set(double p);
 
     std::mutex _m;
     double _p;
@@ -125,25 +100,9 @@ struct progress_simple_stdout_with_time : public progress {
     std::shared_ptr<progress> get() override {
         return std::make_shared<progress_simple_stdout_with_time>();
     }
-    void set(double p) override {
-        _m.lock();
-        _set(p);
-        _m.unlock();
-    };
-
-    void increment(double dp) override {
-        _m.lock();
-        _set(_p + dp);
-        _m.unlock();
-    }
-    virtual void finalize() override {
-        _m.lock();
-        for (uint16_t i = 0; i < (((int)(100 * _p)) / 10); ++i) {
-            std::cout << "=";
-        }
-        std::cout << ">| DONE (" << _t->time() << "s)." << std::endl;
-        _m.unlock();
-    }
+    void set(double p) override;
+    void increment(double dp) override;
+    virtual void finalize() override;
 
     progress_simple_stdout_with_time() : _t(nullptr), _p(0) {
         _t = new timer();
@@ -153,15 +112,7 @@ struct progress_simple_stdout_with_time : public progress {
     }
 
    private:
-    void _set(double p) {  // not synchronized
-        _p = p;
-        for (uint16_t i = 0; i < (((int)(100 * p)) / 10); ++i) {
-            std::cout << "=";
-        }
-        std::cout << "> (" << std::round(100 * p) << "%)";
-        std::cout << "\r";
-        std::cout.flush();
-    };
+    void _set(double p);
 
     timer* _t;
     std::mutex _m;
